
package org.ow2.petals.component.framework.api.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.activation.DataHandler;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/***
 * Contains utility to manipulate MTOM attachments
 * 
 * @author ”Mathieu CARROLLE - mathieu.carrolle@petalslink.com”
 * 
 */
public final class MtomUtil {

    /**
     * Include element namespace
     */
    public static final String MTOM_NSURI = "http://www.w3.org/2004/08/xop/include";

    /**
     * Element : "attachments"
     */
    public static final String NODE_ATTACHMENTS = "attachments";

    /**
     * Compares two strings, the first one being encoded as an URL and not the
     * second.
     * <p>
     * This method is used to compare attachment names.<br />
     * </p>
     * 
     * @param attachmentElementValue
     *            the value of the MTOM attachment, as referenced in the href
     *            attribute
     * @param attachmentName
     *            the attachment name, as specified in the exchange
     * @return true if the arguments are the same (despite the encoding), false
     *         otherwise
     * @throws UnsupportedEncodingException
     */
    public static boolean compare(String attachmentElementValue, String attachmentName) {
        if (attachmentElementValue.equals(attachmentName))
            return true;
        try {
            // In some cases, the attachment ID is encoded as an URL
            String encodedText2 = URLEncoder.encode(attachmentName, "UTF-8");
            if (attachmentElementValue.equals(encodedText2))
                return true;

            // Last attempt
            String encodedText1 = attachmentElementValue.replace("@", "%40");
            if (encodedText1.equals(encodedText2))
                return true;
        } catch (UnsupportedEncodingException e) {

        }
        return false;
    }

    /***
     * Extract element containing xop element.
     * 
     * @param inputDocument
     *            Input xml message.
     * @return The xml element "attachments".
     */
    public static Element extractAttachmentRootElement(Document inputDocument)
            throws MessagingException {
        final Element attachmentsRootElement;
        final NodeList attachmentsCandidatesList = inputDocument.getElementsByTagNameNS("*",
                MtomUtil.NODE_ATTACHMENTS);
        if (attachmentsCandidatesList.getLength() != 1)
            throw new MessagingException("xml element : '" + NODE_ATTACHMENTS + "' not found.");
        attachmentsRootElement = (Element) attachmentsCandidatesList.item(0);
        return attachmentsRootElement;
    }

    /**
     * In the message payload, extract the identifier(cid) associated with each
     * xop:Include element.
     * 
     * @param inputDocument
     *            Input xml message.
     * @param exchange
     * @return List of all cid found in the message
     */
    public static Set<String> extractAttachmentsIdFromPayload(Element attachmentRootElement) {
        final Set<String> attachmentIds = new HashSet<String>();
        String attachmentId = null;
        if (attachmentRootElement.getTextContent() != null)
            attachmentId = attachmentRootElement.getTextContent().trim().replaceFirst("cid:", "");
        // MTOM
        if (attachmentId == null || attachmentId.length() == 0) {
            final NodeList includeNodes = attachmentRootElement.getElementsByTagNameNS(MTOM_NSURI,
                    "*");
            for (int i = 0; i < includeNodes.getLength(); i++) {
                final Node n = includeNodes.item(i);
                if ("include".equalsIgnoreCase(getNodeName(n))) {
                    attachmentId = ((Element) n).getAttribute("href").replaceFirst("cid:", "");
                    attachmentIds.add(attachmentId);
                }
            }
        }
        return attachmentIds;
    }

    /**
     * Build the xml structure according to xop standard.
     * 
     * @param document
     *            Document used to create node.
     * @param contentIds
     *            List of attachment id.
     * @param rootNode
     *            QName representing the root element.
     * @return Return an xml structure according to xop standard. Exemple<br>
     *         "&lt;tns:attachments\&gt;<br>
     *         &lt;xop:Include xmlns:xop="http://www.w3.org/2004/08/xop/include"
     *         href="cid:MyAttachmentId"/&gt;<br>
     *         &lt;/tns:attachments&gt;"
     */
    public static Node generateMtomStructure(Document document, List<String> contentIds,
            QName rootNode) {
        Element rootElt = document.createElementNS(rootNode.getNamespaceURI(),
                getQualifiedName(rootNode));
        for (String cId : contentIds) {
            rootElt.appendChild(generateMtomXopElement(document, cId));
        }
        return rootElt;
    }

    /**
     * Build the xml structure according to xop standard.
     * 
     * @param document
     *            Document used to create node.
     * @param contentIds
     *            List of attachment id.
     * @param rootNode
     *            QName representing the root element.
     * @param customNode
     *            QName representing an additional xml element parent to include for each xop:Include
     *            element
     * @return Return an xml structure according to xop standard. Exemple<br>
     *         &lt;tns:attachments&gt;<br>
     *         <p>&lt;tns:MYELEMENT&gt;&lt;xop:Include
     *         xmlns:xop="http://www.w3.org/2004/08/xop/include"
     *         href="cid:MyAttachmentId"/&gt;&lt;/tns:MYELEMENT&gt;<br>
     *         &lt;tns:MYELEMENT&gt;&lt;xop:Include
     *         xmlns:xop="http://www.w3.org/2004/08/xop/include"
     *         href="cid:MyAttachmentId"/&gt;&lt;/tns:MYELEMENT&gt;<br>
     *         &lt;/tns:attachments&gt;
     */
    public static Node generateMtomStructure(Document document, List<String> contentIds,
            QName rootNode, QName customNode) {
        final Element rootElt = document.createElementNS(rootNode.getNamespaceURI(),
                getQualifiedName(rootNode));
        Element myElt = null;
        for (String cId : contentIds) {
            myElt = document.createElementNS(customNode.getNamespaceURI(),
                    getQualifiedName(customNode));
            myElt.appendChild(generateMtomXopElement(document, cId));
            rootElt.appendChild(myElt);
        }
        return rootElt;
    }

    /***
     * Generate an Include element according to xop standard. Example : <br />
     * &lt;xop:Include xmlns:xop='http://www.w3.org/2004/08/xop/include'
     * href='cid:MYCONTENTID'/>
     * 
     * @param contentId
     * @return the "Include" element
     */
    public static Node generateMtomXopElement(Document doc, String contentId) {
        Element includeElt = doc.createElementNS(MTOM_NSURI, "xop:Include");
        includeElt.setAttribute("href", "cid:" + contentId);
        return includeElt;
    }

    /**
     * Return all attachments by using Xml-binary Optimized Packaging (XOP)
     * standard.
     * 
     * @param inputDocument
     * @param exchange
     * @throw {@link PEtALSCDKException} if :<br>
     *        -no attachment id was found in the message payload<br>
     *        -the attachments id defined in the message payload doesn't match
     *        the attachments id defined in the normalized message<br>
     * @return
     */
    public static Map<String, DataHandler> getAttachments(Document inputDocument,
            NormalizedMessage nmMessage) throws MessagingException {
        final Map<String, DataHandler> mAttachments = new HashMap<String, DataHandler>();

        final Element attachmentRootElement = extractAttachmentRootElement(inputDocument);
        final Set<String> attachmentIds = extractAttachmentsIdFromPayload(attachmentRootElement);
        if (attachmentIds.size() == 0)
            throw new MessagingException("no attachment id was found in the message payload");
        for (String id : attachmentIds) {
            for (Object obj : nmMessage.getAttachmentNames()) {
                final String jbiAttachmentId = (String) obj;
                if (compare(id, jbiAttachmentId))
                    mAttachments.put(jbiAttachmentId, nmMessage.getAttachment(jbiAttachmentId));
            }
        }
        if (mAttachments.size() != attachmentIds.size()) {
            final StringBuilder builder = new StringBuilder();
            builder.append("error when comparing the attachments id" + " | ");
            builder.append("payload attachment id found : " + attachmentIds.toString() + " | ");
            builder.append("normalized message attachment id found : "
                    + nmMessage.getAttachmentNames().toString());
            throw new MessagingException(builder.toString());
        }
        return mAttachments;
    }

    /**
     * Return the node name
     * 
     * @param node
     * @return the name of the XML node (with no name space element)
     */
    public static String getNodeName(Node node) {
        String name = node.getNamespaceURI() != null ? node.getLocalName() : node.getNodeName();
        if (name.contains(":")) { //$NON-NLS-1$
            String[] parts = name.split(":"); //$NON-NLS-1$
            name = parts[parts.length - 1];
        }
        return name;
    }

    /***
     * Return name with prefix if qname contains prefix otherwise return just localpart.
     * 
     * @param qname
     * @return prefix:localpart or localpart
     */
    private final static String getQualifiedName(final QName qname) {
        return qname.getPrefix().length() > 0 ? qname.getPrefix() + ":" + qname.getLocalPart()
                : qname.getLocalPart();
    }
}
